extern struct syscallentry syscall_accept4;
extern struct syscallentry syscall_accept;
extern struct syscallentry syscall_access;
extern struct syscallentry syscall_acct;
extern struct syscallentry syscall_add_key;
extern struct syscallentry syscall_adjtimex;
extern struct syscallentry syscall_alarm;
extern struct syscallentry syscall_arch_prctl;
extern struct syscallentry syscall_bdflush;
extern struct syscallentry syscall_bind;
extern struct syscallentry syscall_brk;
extern struct syscallentry syscall_capget;
extern struct syscallentry syscall_capset;
extern struct syscallentry syscall_chdir;
extern struct syscallentry syscall_chmod;
extern struct syscallentry syscall_chown16;
extern struct syscallentry syscall_chown;
extern struct syscallentry syscall_chroot;
extern struct syscallentry syscall_clock_adjtime;
extern struct syscallentry syscall_clock_getres;
extern struct syscallentry syscall_clock_gettime;
extern struct syscallentry syscall_clock_nanosleep;
extern struct syscallentry syscall_clock_settime;
extern struct syscallentry syscall_clone2;
extern struct syscallentry syscall_clone;
extern struct syscallentry syscall_close;
extern struct syscallentry syscall_connect;
extern struct syscallentry syscall_creat;
extern struct syscallentry syscall_delete_module;
extern struct syscallentry syscall_dup2;
extern struct syscallentry syscall_dup3;
extern struct syscallentry syscall_dup;
extern struct syscallentry syscall_epoll_create1;
extern struct syscallentry syscall_epoll_create;
extern struct syscallentry syscall_epoll_ctl;
extern struct syscallentry syscall_epoll_pwait;
extern struct syscallentry syscall_epoll_wait;
extern struct syscallentry syscall_eventfd2;
extern struct syscallentry syscall_eventfd;
extern struct syscallentry syscall_execve;
extern struct syscallentry syscall_exit;
extern struct syscallentry syscall_exit_group;
extern struct syscallentry syscall_faccessat;
extern struct syscallentry syscall_fadvise64_64;
extern struct syscallentry syscall_fadvise64;
extern struct syscallentry syscall_fallocate;
extern struct syscallentry syscall_fanotify_init;
extern struct syscallentry syscall_fanotify_mark;
extern struct syscallentry syscall_fchdir;
extern struct syscallentry syscall_fchmodat;
extern struct syscallentry syscall_fchmod;
extern struct syscallentry syscall_fchown16;
extern struct syscallentry syscall_fchownat;
extern struct syscallentry syscall_fchown;
extern struct syscallentry syscall_fcntl;
extern struct syscallentry syscall_fdatasync;
extern struct syscallentry syscall_fgetxattr;
extern struct syscallentry syscall_finit_module;
extern struct syscallentry syscall_flistxattr;
extern struct syscallentry syscall_flock;
extern struct syscallentry syscall_fork;
extern struct syscallentry syscall_fremovexattr;
extern struct syscallentry syscall_fsetxattr;
extern struct syscallentry syscall_fstat64;
extern struct syscallentry syscall_fstatat64;
extern struct syscallentry syscall_fstatfs64;
extern struct syscallentry syscall_fstatfs;
extern struct syscallentry syscall_fsync;
extern struct syscallentry syscall_ftruncate64;
extern struct syscallentry syscall_ftruncate;
extern struct syscallentry syscall_futex;
extern struct syscallentry syscall_futimesat;
extern struct syscallentry syscall_getcpu;
extern struct syscallentry syscall_getcwd;
extern struct syscallentry syscall_getdents64;
extern struct syscallentry syscall_getdents;
extern struct syscallentry syscall_getegid16;
extern struct syscallentry syscall_getegid;
extern struct syscallentry syscall_geteuid16;
extern struct syscallentry syscall_geteuid;
extern struct syscallentry syscall_getgid16;
extern struct syscallentry syscall_getgid;
extern struct syscallentry syscall_getgroups16;
extern struct syscallentry syscall_getgroups;
extern struct syscallentry syscall_getitimer;
extern struct syscallentry syscall_get_mempolicy;
extern struct syscallentry syscall_getpagesize;
extern struct syscallentry syscall_getpeername;
extern struct syscallentry syscall_getpgid;
extern struct syscallentry syscall_getpgrp;
extern struct syscallentry syscall_getpid;
extern struct syscallentry syscall_getppid;
extern struct syscallentry syscall_getpriority;
extern struct syscallentry syscall_getresgid16;
extern struct syscallentry syscall_getresgid;
extern struct syscallentry syscall_getresuid16;
extern struct syscallentry syscall_getresuid;
extern struct syscallentry syscall_getrlimit;
extern struct syscallentry syscall_get_robust_list;
extern struct syscallentry syscall_getrusage;
extern struct syscallentry syscall_getsid;
extern struct syscallentry syscall_getsockname;
extern struct syscallentry syscall_getsockopt;
extern struct syscallentry syscall_gettid;
extern struct syscallentry syscall_gettimeofday;
extern struct syscallentry syscall_getuid16;
extern struct syscallentry syscall_getuid;
extern struct syscallentry syscall_getunwind;
extern struct syscallentry syscall_getxattr;
extern struct syscallentry syscall_init_module;
extern struct syscallentry syscall_inotify_add_watch;
extern struct syscallentry syscall_inotify_init1;
extern struct syscallentry syscall_inotify_init;
extern struct syscallentry syscall_inotify_rm_watch;
extern struct syscallentry syscall_io_cancel;
extern struct syscallentry syscall_ioctl;
extern struct syscallentry syscall_io_destroy;
extern struct syscallentry syscall_io_getevents;
extern struct syscallentry syscall_ioperm;
extern struct syscallentry syscall_iopl;
extern struct syscallentry syscall_ioprio_get;
extern struct syscallentry syscall_ioprio_set;
extern struct syscallentry syscall_io_setup;
extern struct syscallentry syscall_io_submit;
extern struct syscallentry syscall_ipc;
extern struct syscallentry syscall_kexec_load;
extern struct syscallentry syscall_keyctl;
extern struct syscallentry syscall_kill;
extern struct syscallentry syscall_lchown16;
extern struct syscallentry syscall_lchown;
extern struct syscallentry syscall_lgetxattr;
extern struct syscallentry syscall_linkat;
extern struct syscallentry syscall_link;
extern struct syscallentry syscall_listen;
extern struct syscallentry syscall_listxattr;
extern struct syscallentry syscall_llistxattr;
extern struct syscallentry syscall_llseek;
extern struct syscallentry syscall_lookup_dcookie;
extern struct syscallentry syscall_lremovexattr;
extern struct syscallentry syscall_lseek;
extern struct syscallentry syscall_lsetxattr;
extern struct syscallentry syscall_lstat64;
extern struct syscallentry syscall_lstat;
extern struct syscallentry syscall_madvise;
extern struct syscallentry syscall_mbind;
extern struct syscallentry syscall_migrate_pages;
extern struct syscallentry syscall_mincore;
extern struct syscallentry syscall_mkdirat;
extern struct syscallentry syscall_mkdir;
extern struct syscallentry syscall_mknodat;
extern struct syscallentry syscall_mknod;
extern struct syscallentry syscall_mlockall;
extern struct syscallentry syscall_mlock;
extern struct syscallentry syscall_mmap2;
extern struct syscallentry syscall_mmap;
extern struct syscallentry syscall_modify_ldt;
extern struct syscallentry syscall_mount;
extern struct syscallentry syscall_move_pages;
extern struct syscallentry syscall_mprotect;
extern struct syscallentry syscall_mq_getsetattr;
extern struct syscallentry syscall_mq_notify;
extern struct syscallentry syscall_mq_open;
extern struct syscallentry syscall_mq_timedreceive;
extern struct syscallentry syscall_mq_timedsend;
extern struct syscallentry syscall_mq_unlink;
extern struct syscallentry syscall_mremap;
extern struct syscallentry syscall_msgctl;
extern struct syscallentry syscall_msgget;
extern struct syscallentry syscall_msgrcv;
extern struct syscallentry syscall_msgsnd;
extern struct syscallentry syscall_msync;
extern struct syscallentry syscall_munlockall;
extern struct syscallentry syscall_munlock;
extern struct syscallentry syscall_munmap;
extern struct syscallentry syscall_name_to_handle_at;
extern struct syscallentry syscall_nanosleep;
extern struct syscallentry syscall_newfstatat;
extern struct syscallentry syscall_newfstat;
extern struct syscallentry syscall_newlstat;
extern struct syscallentry syscall_newstat;
extern struct syscallentry syscall_newuname;
extern struct syscallentry syscall_nfsservctl;
extern struct syscallentry syscall_nice;
extern struct syscallentry syscall_ni_syscall;
extern struct syscallentry syscall_oldreaddir;
extern struct syscallentry syscall_oldumount;
extern struct syscallentry syscall_olduname;
extern struct syscallentry syscall_openat;
extern struct syscallentry syscall_open_by_handle_at;
extern struct syscallentry syscall_open;
extern struct syscallentry syscall_pause;
extern struct syscallentry syscall_pciconfig_read;
extern struct syscallentry syscall_pciconfig_write;
extern struct syscallentry syscall_perf_event_open;
extern struct syscallentry syscall_perfmonctl;
extern struct syscallentry syscall_personality;
extern struct syscallentry syscall_pipe2;
extern struct syscallentry syscall_pipe;
extern struct syscallentry syscall_pivot_root;
extern struct syscallentry syscall_poll;
extern struct syscallentry syscall_ppoll;
extern struct syscallentry syscall_prctl;
extern struct syscallentry syscall_pread64;
extern struct syscallentry syscall_preadv;
extern struct syscallentry syscall_prlimit64;
extern struct syscallentry syscall_pselect6;
extern struct syscallentry syscall_ptrace;
extern struct syscallentry syscall_pwrite64;
extern struct syscallentry syscall_pwritev;
extern struct syscallentry syscall_quotactl;
extern struct syscallentry syscall_readahead;
extern struct syscallentry syscall_read;
extern struct syscallentry syscall_readlinkat;
extern struct syscallentry syscall_readlink;
extern struct syscallentry syscall_readv;
extern struct syscallentry syscall_reboot;
extern struct syscallentry syscall_recv;
extern struct syscallentry syscall_recvfrom;
extern struct syscallentry syscall_recvmmsg;
extern struct syscallentry syscall_recvmsg;
extern struct syscallentry syscall_remap_file_pages;
extern struct syscallentry syscall_removexattr;
extern struct syscallentry syscall_renameat;
extern struct syscallentry syscall_rename;
extern struct syscallentry syscall_request_key;
extern struct syscallentry syscall_restart_syscall;
extern struct syscallentry syscall_rmdir;
extern struct syscallentry syscall_rtas;
extern struct syscallentry syscall_rt_sigaction;
extern struct syscallentry syscall_rt_sigpending;
extern struct syscallentry syscall_rt_sigprocmask;
extern struct syscallentry syscall_rt_sigqueueinfo;
extern struct syscallentry syscall_rt_sigreturn;
extern struct syscallentry syscall_rt_sigsuspend;
extern struct syscallentry syscall_rt_sigtimedwait;
extern struct syscallentry syscall_rt_tgsigqueueinfo;
extern struct syscallentry syscall_sched_getaffinity;
extern struct syscallentry syscall_sched_getparam;
extern struct syscallentry syscall_sched_get_priority_max;
extern struct syscallentry syscall_sched_get_priority_min;
extern struct syscallentry syscall_sched_getscheduler;
extern struct syscallentry syscall_sched_rr_get_interval;
extern struct syscallentry syscall_sched_setaffinity;
extern struct syscallentry syscall_sched_setparam;
extern struct syscallentry syscall_sched_setscheduler;
extern struct syscallentry syscall_sched_yield;
extern struct syscallentry syscall_select;
extern struct syscallentry syscall_semctl;
extern struct syscallentry syscall_semget;
extern struct syscallentry syscall_semop;
extern struct syscallentry syscall_semtimedop;
extern struct syscallentry syscall_send;
extern struct syscallentry syscall_sendfile64;
extern struct syscallentry syscall_sendfile;
extern struct syscallentry syscall_sendmsg;
extern struct syscallentry syscall_sendto;
extern struct syscallentry syscall_setdomainname;
extern struct syscallentry syscall_setfsgid16;
extern struct syscallentry syscall_setfsgid;
extern struct syscallentry syscall_setfsuid16;
extern struct syscallentry syscall_setfsuid;
extern struct syscallentry syscall_setgid16;
extern struct syscallentry syscall_setgid;
extern struct syscallentry syscall_setgroups16;
extern struct syscallentry syscall_setgroups;
extern struct syscallentry syscall_sethostname;
extern struct syscallentry syscall_setitimer;
extern struct syscallentry syscall_set_mempolicy;
extern struct syscallentry syscall_setpgid;
extern struct syscallentry syscall_setpriority;
extern struct syscallentry syscall_setregid16;
extern struct syscallentry syscall_setregid;
extern struct syscallentry syscall_setresgid16;
extern struct syscallentry syscall_setresgid;
extern struct syscallentry syscall_setresuid16;
extern struct syscallentry syscall_setresuid;
extern struct syscallentry syscall_setreuid16;
extern struct syscallentry syscall_setreuid;
extern struct syscallentry syscall_setrlimit;
extern struct syscallentry syscall_set_robust_list;
extern struct syscallentry syscall_setsid;
extern struct syscallentry syscall_setsockopt;
extern struct syscallentry syscall_set_tid_address;
extern struct syscallentry syscall_settimeofday;
extern struct syscallentry syscall_setuid16;
extern struct syscallentry syscall_setuid;
extern struct syscallentry syscall_setxattr;
extern struct syscallentry syscall_sgetmask;
extern struct syscallentry syscall_shmat;
extern struct syscallentry syscall_shmctl;
extern struct syscallentry syscall_shmdt;
extern struct syscallentry syscall_shmget;
extern struct syscallentry syscall_shutdown;
extern struct syscallentry syscall_sigaction;
extern struct syscallentry syscall_sigaltstack;
extern struct syscallentry syscall_signal;
extern struct syscallentry syscall_signalfd4;
extern struct syscallentry syscall_signalfd;
extern struct syscallentry syscall_sigpending;
extern struct syscallentry syscall_sigprocmask;
extern struct syscallentry syscall_sigreturn;
extern struct syscallentry syscall_sigsuspend;
extern struct syscallentry syscall_socket;
extern struct syscallentry syscall_socketcall;
extern struct syscallentry syscall_socketpair;
extern struct syscallentry syscall_splice;
extern struct syscallentry syscall_ssetmask;
extern struct syscallentry syscall_stat64;
extern struct syscallentry syscall_stat;
extern struct syscallentry syscall_statfs64;
extern struct syscallentry syscall_statfs;
extern struct syscallentry syscall_stime;
extern struct syscallentry syscall_swapoff;
extern struct syscallentry syscall_swapon;
extern struct syscallentry syscall_symlinkat;
extern struct syscallentry syscall_symlink;
extern struct syscallentry syscall_sync;
extern struct syscallentry syscall_sync_file_range2;
extern struct syscallentry syscall_sync_file_range;
extern struct syscallentry syscall_syncfs;
extern struct syscallentry syscall_sysctl;
extern struct syscallentry syscall_sysfs;
extern struct syscallentry syscall_sysinfo;
extern struct syscallentry syscall_syslog;
extern struct syscallentry syscall_tee;
extern struct syscallentry syscall_tgkill;
extern struct syscallentry syscall_time;
extern struct syscallentry syscall_timer_create;
extern struct syscallentry syscall_timer_delete;
extern struct syscallentry syscall_timerfd_create;
extern struct syscallentry syscall_timerfd_gettime;
extern struct syscallentry syscall_timerfd_settime;
extern struct syscallentry syscall_timer_getoverrun;
extern struct syscallentry syscall_timer_gettime;
extern struct syscallentry syscall_timer_settime;
extern struct syscallentry syscall_times;
extern struct syscallentry syscall_tkill;
extern struct syscallentry syscall_truncate64;
extern struct syscallentry syscall_truncate;
extern struct syscallentry syscall_umask;
extern struct syscallentry syscall_umount;
extern struct syscallentry syscall_uname;
extern struct syscallentry syscall_unlinkat;
extern struct syscallentry syscall_unlink;
extern struct syscallentry syscall_unshare;
extern struct syscallentry syscall_uselib;
extern struct syscallentry syscall_ustat;
extern struct syscallentry syscall_utime;
extern struct syscallentry syscall_utimensat;
extern struct syscallentry syscall_utimes;
extern struct syscallentry syscall_vfork;
extern struct syscallentry syscall_vhangup;
extern struct syscallentry syscall_vm86;
extern struct syscallentry syscall_vm86old;
extern struct syscallentry syscall_vmsplice;
extern struct syscallentry syscall_wait4;
extern struct syscallentry syscall_waitid;
extern struct syscallentry syscall_waitpid;
extern struct syscallentry syscall_write;
extern struct syscallentry syscall_writev;
extern struct syscallentry syscall_sendmmsg;
extern struct syscallentry syscall_setns;
extern struct syscallentry syscall_process_vm_readv;
extern struct syscallentry syscall_process_vm_writev;
extern struct syscallentry syscall_kcmp;
extern struct syscallentry syscall_pciconfig_iobase;
extern struct syscallentry syscall_arm_fadvise64_64;
extern struct syscallentry syscall_sh_cacheflush;
extern struct syscallentry syscall_sched_setattr;
extern struct syscallentry syscall_sched_getattr;
extern struct syscallentry syscall_renameat2;
extern struct syscallentry syscall_kern_features;
extern struct syscallentry syscall_getrandom;
extern struct syscallentry syscall_seccomp;
extern struct syscallentry syscall_memfd_create;
extern struct syscallentry syscall_kexec_file_load;
extern struct syscallentry syscall_bpf;
extern struct syscallentry syscall_execveat;
extern struct syscallentry syscall_userfaultfd;
extern struct syscallentry syscall_membarrier;
extern struct syscallentry syscall_mlock2;
extern struct syscallentry syscall_copy_file_range;
extern struct syscallentry syscall_preadv2;
extern struct syscallentry syscall_pwritev2;
extern struct syscallentry syscall_pkey_mprotect;
extern struct syscallentry syscall_pkey_alloc;
extern struct syscallentry syscall_pkey_free;
extern struct syscallentry syscall_statx;
extern struct syscallentry syscall_io_pgetevents;
extern struct syscallentry syscall_rseq;
extern struct syscallentry syscall_pidfd_send_signal;
extern struct syscallentry syscall_io_uring_setup;
extern struct syscallentry syscall_io_uring_enter;
extern struct syscallentry syscall_io_uring_register;
extern struct syscallentry syscall_open_tree;
extern struct syscallentry syscall_move_mount;
extern struct syscallentry syscall_fsopen;
extern struct syscallentry syscall_fsconfig;
extern struct syscallentry syscall_fsmount;
extern struct syscallentry syscall_fspick;
extern struct syscallentry syscall_pidfd_open;
extern struct syscallentry syscall_clone3;
extern struct syscallentry syscall_close_range;
extern struct syscallentry syscall_openat2;
extern struct syscallentry syscall_pidfd_getfd;
extern struct syscallentry syscall_faccessat2;
extern struct syscallentry syscall_process_madvise;
extern struct syscallentry syscall_epoll_pwait2;
extern struct syscallentry syscall_mount_setattr;
extern struct syscallentry syscall_quotactl_fd;
extern struct syscallentry syscall_landlock_create_ruleset;
extern struct syscallentry syscall_landlock_add_rule;
extern struct syscallentry syscall_landlock_restrict_self;
extern struct syscallentry syscall_memfd_secret;
extern struct syscallentry syscall_process_mrelease;
extern struct syscallentry syscall_futex_waitv;
extern struct syscallentry syscall_set_mempolicy_home_node;

extern struct syscallentry syscall_runtime_instr;
extern struct syscallentry syscall_s390_pci_mmio_write;
extern struct syscallentry syscall_s390_pci_mmio_read;
extern struct syscallentry syscall_s390_guarded_storage;
extern struct syscallentry syscall_s390_sthyi;

extern struct syscallentry syscall_riscv_flush_icache;

unsigned int random_fcntl_setfl_flags(void);
